@@ -4,10 +4,12 @@ packing.py
 
 Pack rectangular regions onto larger rectangular regions.
 """
+
 import numpy as np
 
-from ..util import allclose, bounds_tree
 from ..constants import log, tol
+from ..typed import ArrayLike, Integer, NDArray, Number, Optional, float64
+from ..util import allclose, bounds_tree
 
 # floating point zero
 _TOL_ZERO = 1e-12
@@ -71,7 +73,6 @@ class RectangleBin:
           Position of insertion in the tree or None
           if the insertion was unsuccessful.
         """
-
         for child in self.child:
             # try inserting into child cells
             attempt = child.insert(size=size, rotate=rotate)
@@ -120,7 +121,7 @@ class RectangleBin:
         # split hyper-rectangle along axis
         # note that split is *absolute* distance not offset
         # so we have to add the current min to the size
-        splits = np.tile(bounds, (2, 1))
+        splits = np.vstack((bounds, bounds))
         splits[1:3, axis] = bounds[0][axis] + size[axis]
 
         # assign two children
@@ -159,7 +160,7 @@ def _roll(a, count):
         return np.concatenate([a[-count:], a[:-count]])
 
 
-def rectangles_single(extents, size=None, shuffle=False, rotate=True):
+def rectangles_single(extents, size=None, shuffle=False, rotate=True, random=None):
     """
     Execute a single insertion order of smaller rectangles onto
     a larger rectangle using a binary space partition tree.
@@ -169,23 +170,25 @@ def rectangles_single(extents, size=None, shuffle=False, rotate=True):
     extents : (n, dimension) float
       The size of the hyper-rectangles to pack.
     size : None or (dim,) float
-      Maximum size of container to pack onto. If not passed it
-      will re-root the tree when large items are inserted.
+      Maximum size of container to pack onto.
+      If not passed it will re-root the tree when items
+      larger than any available node are inserted.
     shuffle : bool
       Whether or not to shuffle the insert order of the
       smaller rectangles, as the final packing density depends
       on insertion order.
     rotate : bool
-      If True, allow rotation.
+      If True, allow integer-roll rotation.
 
     Returns
     ---------
-    bounds : (n, 2, dim) float
+    bounds : (m, 2, dim) float
       Axis aligned resulting bounds in space
     transforms : (m, dim + 1, dim + 1) float
-      Homogenous transformation including rotation.
-    inserted : (n,) bool
-      Which of the original rectangles were packed
+      Homogeneous transformation including rotation.
+    consume : (n,) bool
+      Which of the original rectangles were packed,
+      i.e. `consume.sum() == m`
     """
 
     extents = np.asanyarray(extents, dtype=np.float64)
@@ -197,15 +200,17 @@ def rectangles_single(extents, size=None, shuffle=False, rotate=True):
     order = np.argsort(extents.max(axis=1))[::-1]
 
     if shuffle:
-        # reorder with permutations
-        order = np.random.permutation(order)
+        if random is not None:
+            order = random.permutation(order)
+        else:
+            # reorder with permutations
+            order = np.random.permutation(order)
 
     if size is None:
         # if no bounds are passed start it with the size of a large
         # rectangle exactly which will require re-rooting for
         # subsequent insertions
-        root_bounds = [[0.0] * dimension,
-                       extents[extents.ptp(axis=1).argmax()]]
+        root_bounds = [[0.0] * dimension, extents[np.ptp(extents, axis=1).argmax()]]
     else:
         # restrict the bounds to passed size and disallow re-rooting
         root_bounds = [[0.0] * dimension, size]
@@ -225,7 +230,7 @@ def rectangles_single(extents, size=None, shuffle=False, rotate=True):
             # get the size of the current root node
             bounds = root.bounds
             # current extents
-            current = bounds.ptp(axis=0)
+            current = np.ptp(bounds, axis=0)
 
             # pick the direction which has the least hyper-volume.
             best = np.inf
@@ -240,7 +245,7 @@ def rectangles_single(extents, size=None, shuffle=False, rotate=True):
                 np.fill_diagonal(ch, stack.sum(axis=0))
 
                 # choose the new AABB by which one minimizes hyper-volume
-                choice_prod = np.product(ch, axis=1)
+                choice_prod = np.prod(ch, axis=1)
                 if choice_prod.min() < best:
                     choices = ch
                     choices_idx = choice_prod.argmin()
@@ -272,12 +277,10 @@ def rectangles_single(extents, size=None, shuffle=False, rotate=True):
             # this node has children so it is occupied
             new_root.occupied = True
             # create a bin for both bounds
-            new_root.child = [RectangleBin(bounds_ori),
-                              RectangleBin(bounds_ins)]
+            new_root.child = [RectangleBin(bounds_ori), RectangleBin(bounds_ins)]
 
             # insert the original sheet into the new tree
-            root_offset = new_root.child[0].insert(
-                bounds.ptp(axis=0), rotate=rotate)
+            root_offset = new_root.child[0].insert(np.ptp(bounds, axis=0), rotate=rotate)
             # we sized the cells so original tree would fit
             assert root_offset is not None
 
@@ -319,9 +322,13 @@ def paths(paths, **kwargs):
     Returns
     ------------
     packed : trimesh.path.Path2D
-      Object containing input geometry
-    inserted : (m,) int
-      Indexes of paths inserted into result
+      All paths packed into a single path object.
+    transforms : (m, 3, 3) float
+      Homogeneous transforms to move paths from their
+      original position to the new one.
+    consume : (n,) bool
+      Which of the original paths were inserted,
+      i.e. `consume.sum() == m`
     """
     from .util import concatenate
 
@@ -330,13 +337,12 @@ def paths(paths, **kwargs):
     packable = []
     original = []
     for index, path in enumerate(paths):
-        quantity = path.metadata.get('quantity', 1)
+        quantity = path.metadata.get("quantity", 1)
         original.extend([index] * quantity)
         packable.extend([path.polygons_closed[path.root[0]]] * quantity)
 
     # pack the polygons using rectangular bin packing
-    transforms, consume = polygons(
-        polygons=packable, **kwargs)
+    transforms, consume = polygons(polygons=packable, **kwargs)
 
     positioned = []
     for index, matrix in zip(np.nonzero(consume)[0], transforms):
@@ -347,12 +353,10 @@ def paths(paths, **kwargs):
     # append all packed paths into a single Path object
     packed = concatenate(positioned)
 
-    return packed, consume
+    return packed, transforms, consume
 
 
-def polygons(polygons,
-             size=None,
-             **kwargs):
+def polygons(polygons, **kwargs):
     """
     Pack polygons into a rectangle by taking each Polygon's OBB
     and then packing that as a rectangle.
@@ -361,84 +365,82 @@ def polygons(polygons,
     ------------
     polygons : (n,) shapely.geometry.Polygon
       Source geometry
-    size : (2,) float
-      Size of rectangular sheet
     **kwargs : dict
       Passed through to `packing.rectangles`.
 
     Returns
     -------------
-    overall_inserted : (m,) int
-      Indexes of inserted polygons
-    packed : (m, 3, 3) float
+    transforms : (m, 3, 3) float
       Homogeonous transforms from original frame to
       packed frame.
+    consume : (n,) bool
+      Which of the original polygons was packed,
+      i.e. `consume.sum() == m`
     """
 
-    from .polygons import polygons_obb, polygon_bounds
+    from .polygons import polygon_bounds, polygons_obb
 
     # find the oriented bounding box of the polygons
     obb, extents = polygons_obb(polygons)
 
-    spacing = kwargs.get('spacing', 0.0)
-    # pad all sides of the rectangle
-    extents += 2.0 * spacing
-    # move the OBB transform so the polygon is centered
-    # in the padded rectangle
-
     # run packing for a number of iterations
-    bounds, consume = rectangles(
-        extents=extents,
-        size=size,
-        **kwargs)
+    bounds, consume = rectangles(extents=extents, **kwargs)
 
-    assert not bounds_overlap(bounds)
-    log.debug('%i/%i parts were packed successfully',
-              consume.sum(), len(polygons))
+    log.debug("%i/%i parts were packed successfully", consume.sum(), len(polygons))
 
     # transformations to packed positions
     roll = roll_transform(bounds=bounds, extents=extents[consume])
-    # add the spacing offset
-    roll[:, :2, 2] += spacing
 
-    transforms = np.array([np.dot(b, a) for a, b in
-                           zip(obb[consume], roll)])
+    transforms = np.array([np.dot(b, a) for a, b in zip(obb[consume], roll)])
 
     if tol.strict:
+        # original bounds should not overlap
+        assert not bounds_overlap(bounds)
+        # confirm transfor
         check_bound = np.array(
-            [polygon_bounds(polygons[index], matrix=m)
-             for index, m in zip(np.nonzero(consume)[0], transforms)])
+            [
+                polygon_bounds(polygons[index], matrix=m)
+                for index, m in zip(np.nonzero(consume)[0], transforms)
+            ]
+        )
         assert not bounds_overlap(check_bound)
 
     return transforms, consume
 
 
-def rectangles(extents,
-               size=None,
-               density_escape=0.9,
-               spacing=0.0,
-               iterations=50,
-               rotate=True,
-               quanta=None):
+def rectangles(
+    extents,
+    size=None,
+    density_escape=0.99,
+    spacing=None,
+    iterations=50,
+    rotate=True,
+    quanta=None,
+    seed=None,
+):
     """
-    Run multiple iterations of rectangle packing.
+    Run multiple iterations of rectangle packing, this is the
+    core function for all rectangular packing.
 
     Parameters
     ------------
     extents : (n, dimension) float
       Size of hyper-rectangle to be packed
     size : None or (dimension,) float
-      Size of sheet to pack onto
+      Size of sheet to pack onto. If not passed tree will be allowed
+      to create new volume-minimizing parent nodes.
     density_escape : float
-      Exit early if density is above this threshold
+      Exit early if rectangular density is above this threshold.
     spacing : float
-      Distance to allow between rect
+      Distance to allow between rectangles
     iterations : int
       Number of iterations to run
     rotate : bool
       Allow right angle rotations or not.
     quanta : None or float
       Discrete "snap" interval.
+    seed
+      If deterministic results are needed seed the RNG here.
 
     Returns
     ---------
@@ -447,48 +449,72 @@ def rectangles(extents,
     inserted : (n,) bool
       Which of the original rect were packed.
     """
-
-    extents = np.array(extents)
+    # copy extents and make sure they are floats
+    extents = np.array(extents, dtype=np.float64)
     dim = extents.shape[1]
 
+    if spacing is not None:
+        # add on any requested spacing
+        extents += spacing * 2.0
+
     # hyper-volume: area in 2D, volume in 3D, party in 4D
-    area = np.product(extents, axis=1)
+    area = np.prod(extents, axis=1)
     # best density percentage in 0.0 - 1.0
     best_density = 0.0
     # how many rect were inserted
     best_count = 0
 
+    if seed is None:
+        random = None
+    else:
+        random = np.random.default_rng(seed=seed)
+
     for i in range(iterations):
         # run a single insertion order
         # don't shuffle the first run, shuffle subsequent runs
         bounds, insert = rectangles_single(
-            extents=extents, size=size, shuffle=(i != 0))
+            extents=extents, size=size, shuffle=(i != 0), rotate=rotate, random=random
+        )
 
         count = insert.sum()
-        extents_all = bounds.reshape((-1, dim)).ptp(axis=0)
+        extents_all = np.ptp(bounds.reshape((-1, dim)), axis=0)
 
         if quanta is not None:
             # compute the density using an upsized quanta
             extents = np.ceil(extents_all / quanta) * quanta
 
         # calculate the packing density
-        density = area[insert].sum() / np.product(extents_all)
+        density = area[insert].sum() / np.prod(extents_all)
 
         # compare this packing density against our best
         if density > best_density or count > best_count:
             best_density = density
             best_count = count
             # save the result
-            result = (bounds, insert)
+            result = [bounds, insert]
             # exit early if everything is inserted and
             # we have exceeded our target density
             if density > density_escape and insert.all():
                 break
 
+    if spacing is not None:
+        # shrink the bounds by spacing
+        result[0] += [[[spacing], [-spacing]]]
+
+    log.debug(f"{iterations} iterations packed with density {best_density:0.3f}")
+
     return result
 
 
-def images(images, power_resize=False):
+def images(
+    images,
+    power_resize: bool = False,
+    deduplicate: bool = False,
+    iterations: Optional[Integer] = 50,
+    seed: Optional[Integer] = None,
+    spacing: Optional[Number] = None,
+    mode: Optional[str] = None,
+):
     """
     Pack a list of images and return result and offsets.
 
@@ -500,6 +526,13 @@ def images(images, power_resize=False):
       Should the result image be upsized to the nearest
       power of two? Not every GPU supports materials that
       aren't a power of two size.
+    deduplicate
+      Should images that have identical hashes be inserted
+      more than once?
+    mode
+      If passed return an output image with the
+      requested mode, otherwise will be picked
+      from the input images.
 
     Returns
     -----------
@@ -510,54 +543,159 @@ def images(images, power_resize=False):
     """
     from PIL import Image
 
-    # use the number of pixels as the rectangle size
-    bounds, insert = rectangles(
-        extents=[i.size for i in images], rotate=False)
-    # really should have inserted all the rect
-    assert insert.all()
+    if deduplicate:
+        # only pack duplicate images once
+        _, index, inverse = np.unique(
+            [hash(i.tobytes()) for i in images], return_index=True, return_inverse=True
+        )
+        # use the number of pixels as the rectangle size
+        bounds, insert = rectangles(
+            extents=[images[i].size for i in index],
+            rotate=False,
+            iterations=iterations,
+            seed=seed,
+            spacing=spacing,
+        )
+        # really should have inserted all the rect
+        assert insert.all()
+        # re-index bounds back to original indexes
+        bounds = bounds[inverse]
+        assert np.allclose(np.ptp(bounds, axis=1), [i.size for i in images])
+    else:
+        # use the number of pixels as the rectangle size
+        bounds, insert = rectangles(
+            extents=[i.size for i in images],
+            rotate=False,
+            iterations=iterations,
+            seed=seed,
+            spacing=spacing,
+        )
+        # really should have inserted all the rect
+        assert insert.all()
+
+    if spacing is None:
+        spacing = 0
+    else:
+        spacing = int(spacing)
 
     # offsets should be integer multiple of pizels
     offset = bounds[:, 0].round().astype(int)
-
-    extents = bounds.reshape((-1, 2)).ptp(axis=0)
+    extents = np.ptp(bounds.reshape((-1, 2)), axis=0) + (spacing * 2)
     size = extents.round().astype(int)
     if power_resize:
         # round up all dimensions to powers of 2
         size = (2 ** np.ceil(np.log2(size))).astype(np.int64)
 
-    # create the image
-    result = Image.new('RGB', tuple(size))
+    if mode is None:
+        # get the mode of every input image
+        modes = list({i.mode for i in images})
+        # pick the longest mode as a simple heuristic
+        # which prefers "RGBA" over "RGB"
+        mode = modes[np.argmax([len(m) for m in modes])]
+
+    # create the image in the mode of the first image
+    result = Image.new(mode, tuple(size))
+
+    done = set()
     # paste each image into the result
     for img, off in zip(images, offset):
-        result.paste(img, tuple(off))
+        if tuple(off) not in done:
+            # box is upper left corner
+            corner = (off[0], size[1] - img.size[1] - off[1])
+            result.paste(img, box=corner)
+        else:
+            done.add(tuple(off))
 
     return result, offset
 
 
-def visualize(extents, bounds, meshes=None):
+def meshes(meshes, **kwargs):
+    """
+    Pack 3D meshes into a rectangular volume using box packing.
+
+    Parameters
+    ------------
+    meshes : (n,) trimesh.Trimesh
+      Input geometry to pack
+    **kwargs : dict
+      Passed to `packing.rectangles`
+
+    Returns
+    ------------
+    placed : (m,) trimesh.Trimesh
+      Meshes moved into the rectangular volume.
+    transforms : (m, 4, 4) float
+      Homogeneous transform moving mesh from original
+      position to being packed in a rectangular volume.
+    consume : (n,) bool
+      Which of the original meshes were inserted,
+      i.e. `consume.sum() == m`
+    """
+    # pack meshes relative to their oriented bounding boxes
+    obbs = [i.bounding_box_oriented for i in meshes]
+    obb_extent = np.array([i.primitive.extents for i in obbs])
+    obb_transform = np.array([o.primitive.transform for o in obbs])
+
+    # run packing
+    bounds, consume = rectangles(obb_extent, **kwargs)
+
+    # generate the transforms from an origin centered AABB
+    # to the final placed and rotated AABB
+    transforms = np.array(
+        [
+            np.dot(r, np.linalg.inv(o))
+            for o, r in zip(
+                obb_transform[consume],
+                roll_transform(bounds=bounds, extents=obb_extent[consume]),
+            )
+        ],
+        dtype=np.float64,
+    )
+
+    # copy the meshes and move into position
+    placed = [
+        meshes[index].copy().apply_transform(T)
+        for index, T in zip(np.nonzero(consume)[0], transforms)
+    ]
+
+    return placed, transforms, consume
+
+
+def visualize(extents, bounds):
     """
     Visualize a 3D box packing.
+
+    Parameters
+    ------------
+    extents : (n, 3) float
+      AABB size before packing.
+    bounds : (n, 2, 3) float
+      AABB location after packing.
+
+    Returns
+    ------------
+    scene : trimesh.Scene
+      Scene with boxes at requested locations.
     """
     from ..creation import box
-    from ..visual import random_color
     from ..scene import Scene
+    from ..visual import random_color
 
+    # use a roll transform to verify extents
     transforms = roll_transform(bounds=bounds, extents=extents)
-    if meshes is None:
-        meshes = [box(extents=e) for e in extents]
-    collect = []
-    for ori, matrix in zip(meshes, transforms):
-        m = ori.copy()
+    meshes = [box(extents=e) for e in extents]
+
+    for m, matrix, check in zip(meshes, transforms, bounds):
         m.apply_transform(matrix)
+        assert np.allclose(m.bounds, check)
         m.visual.face_colors = random_color()
-        collect.append(m)
-    return Scene(collect)
+    return Scene(meshes)
 
 
-def roll_transform(bounds, extents):
+def roll_transform(bounds: ArrayLike, extents: ArrayLike) -> NDArray[float64]:
     """
     Packing returns rotations with integer "roll" which
-    needs to be converted into a homogenous rotation matrix.
+    needs to be converted into a homogeneous rotation matrix.
 
     Currently supports `dimension=2` and `dimension=3`.
 
@@ -572,16 +710,16 @@ def roll_transform(bounds, extents):
     Returns
     ----------
     transforms : (n, dimension + 1, dimension + 1) float
-      Homogenous transformation to move cuboid at the origin
+      Homogeneous transformation to move cuboid at the origin
       into the position determined by `bounds`.
     """
     if len(bounds) != len(extents):
-        raise ValueError('`bounds` must match `extents`')
+        raise ValueError("`bounds` must match `extents`")
     if len(extents) == 0:
         return []
 
     # find the size of the AABB of the passed bounds
-    passed = bounds.ptp(axis=1)
+    passed = np.ptp(bounds, axis=1)
     # zeroth index is 2D, `1` is 3D
     dimension = passed.shape[1]
 
@@ -591,19 +729,28 @@ def roll_transform(bounds, extents):
     # a lookup table for rotations for rolling cuboiods
     # as `lookup[dimension - 2][roll]`
     # implemented for 2D and 3D
-    lookup = [np.array([np.eye(3),
-                        np.array([[0., -1., 0.],
-                                  [1., 0., 0.],
-                                  [0., 0., 1.]])]),
-              np.array([np.eye(4),
-                        [[-0., -0., -1., -0.],
-                         [-1., -0., -0., -0.],
-                         [0., 1., 0., 0.],
-                         [0., 0., 0., 1.]],
-                        [[-0., -1., -0., -0.],
-                         [0., 0., 1., 0.],
-                         [-1., -0., -0., -0.],
-                         [0., 0., 0., 1.]]])]
+    lookup = [
+        np.array(
+            [np.eye(3), np.array([[0.0, -1.0, 0.0], [1.0, 0.0, 0.0], [0.0, 0.0, 1.0]])]
+        ),
+        np.array(
+            [
+                np.eye(4),
+                [
+                    [-0.0, -0.0, -1.0, -0.0],
+                    [-1.0, -0.0, -0.0, -0.0],
+                    [0.0, 1.0, 0.0, 0.0],
+                    [0.0, 0.0, 0.0, 1.0],
+                ],
+                [
+                    [-0.0, -1.0, -0.0, -0.0],
+                    [0.0, 0.0, 1.0, 0.0],
+                    [-1.0, -0.0, -0.0, -0.0],
+                    [0.0, 0.0, 0.0, 1.0],
+                ],
+            ]
+        ),
+    ]
 
     # rectangular rotation involves rolling
     for roll in range(extents.shape[1]):
@@ -612,7 +759,7 @@ def roll_transform(bounds, extents):
         rolled = np.roll(extents, roll, axis=1)
         # check to see if the rolled original extents
         # match the requested bounding box
-        ok = (passed - rolled).ptp(axis=1) < _TOL_ZERO
+        ok = np.ptp((passed - rolled), axis=1) < _TOL_ZERO
         if not ok.any():
             continue
 
@@ -627,17 +774,21 @@ def roll_transform(bounds, extents):
         if dimension == 3:
             # make sure bounds match inputs
             from ..creation import box
-            assert all(allclose(box(extents=e).apply_transform(m).bounds, b)
-                       for b, e, m in zip(bounds, extents, result))
+
+            assert all(
+                allclose(box(extents=e).apply_transform(m).bounds, b)
+                for b, e, m in zip(bounds, extents, result)
+            )
         elif dimension == 2:
             # in 2D check with a rectangle
             from .creation import rectangle
+
             assert all(
-                allclose(rectangle(
-                    bounds=[-e / 2, e / 2]).apply_transform(m).bounds, b)
-                for b, e, m in zip(bounds, extents, result))
+                allclose(rectangle(bounds=[-e / 2, e / 2]).apply_transform(m).bounds, b)
+                for b, e, m in zip(bounds, extents, result)
+            )
         else:
-            raise ValueError('unsupported dimension')
+            raise ValueError("unsupported dimension")
 
     return result
 
@@ -661,9 +812,9 @@ def bounds_overlap(bounds, epsilon=1e-8):
       True if any bound intersects any other bound.
     """
     # pad AABB by epsilon for deterministic intersections
-    padded = np.array(bounds) + np.reshape(
-        [epsilon, -epsilon], (1, 2, 1))
+    padded = np.array(bounds) + np.reshape([epsilon, -epsilon], (1, 2, 1))
     tree = bounds_tree(padded)
     # every returned AABB should not overlap with any other AABB
-    return any(set(tree.intersection(current.ravel())) !=
-               {i} for i, current in enumerate(bounds))
+    return any(
+        set(tree.intersection(current.ravel())) != {i} for i, current in enumerate(bounds)
+    )
